% Input :
% 		 kernelName : which kernel to get features for
%		 kArgs : arguments for the kernel (sigma^2 for exponential, degree for polynomial)
%		 numFeatures : how many features do we want ?
%		 origDimensions : what is the original dimensionality of the problem
%		 maxFeatureComplexity : what is the maximum complexity of a feature
%		 version : do we want to implement the heuristic which does not generate features for the first two terms at all ?

% Output : featureConstructor : a structure with the following fields
%							    kernelName : name of kernel for which the feature constructors are being generated
%							    kArgs : details of the kernel (degree for polynomial kernel and sigma for exponential kernel)
%							    d : original dimensionality
%							    D : desired dimensionality
%							    N : a D-dimensional vector containing the complexities of each of the D features
%							    maxFeatureComplexity : the maximum complexity allowed for a feature
%							    a : a D-dimensional vector containing the scalar coefficients of each of the D features
%							    numFeatureConstructors : total number of feature constructors - a feature may require more than one feature constructor
%							    W : a binary matrix of d x numFeatureConstructors dimensionality having values in {-1,+1} that constitutes the feature constructors


function [featureConstructors] = getKKFeatureConstructors(kernelName,kArgs,numFeatures,origDimensions,maxFeatureComplexity,version)
	featureConstructors.maxFeatureComplexity = maxFeatureComplexity;
	featureConstructors.d = origDimensions;
	featureConstructors.D = numFeatures;
	featureConstructors.kernelName = kernelName;
	featureConstructors.kArgs = kArgs;
	
	% Get feature complexities for all the features and set the limit to the maximum complexity specified
	featureConstructors.N = floor(log2(1./rand(featureConstructors.D,1)));
	
	% The curious case of homogeneous polynomial kernel
	switch(kernelName),
		case 'homopoly',
			% No point in having version 1 in case of homogeneous polynomial kernel
			version  = 0;
			% The expansion in polynomial kernels cant exceed the degree of the kernel
			featureConstructors.maxFeatureComplexity = min(featureConstructors.maxFeatureComplexity,kArgs);
			% All features must have the same complexity
			featureConstructors.N = featureConstructors.maxFeatureComplexity*ones(featureConstructors.D,1);
		case 'poly'
			% The expansion in polynomial kernels cant exceed the degree of the kernel
			featureConstructors.maxFeatureComplexity = min(featureConstructors.maxFeatureComplexity,kArgs);
		case 'exp',
			% Nothing to be done
		otherwise,
			fprintf('Unknown Kernel type\n');
			keyboard;
	end
	
	switch(version),
		case 0,
			% Nothing to be done
		case 1,
			% Dont generate features for the first two terms, they will be thrown in explicitly later
			featureConstructors.N = featureConstructors.N + 2;
		otherwise,
			fprintf('Unknown version\n');
			keyboard;
	end

	featureConstructors.N(featureConstructors.N > featureConstructors.maxFeatureComplexity) = featureConstructors.maxFeatureComplexity;
	
	% Get the feature coefficients
	switch(kernelName),
		% <x,y>^p
		case 'homopoly',
			% All the coefficients are just one
			featureConstructors.a = ones(featureConstructors.D,1);
		% (1 + <x,y>)^p
		case 'poly'
			% Coefficients are 2^(N+1)*(p,N)
			featureConstructors.a = sqrt(factorial(kArgs)*2.^(featureConstructors.N+1)./(factorial(featureConstructors.N).*factorial(kArgs - featureConstructors.N)));
		% exp(<x,y>/sigma)
		case 'exp',
			% Coefficients are 2^(N+1)/(sigma^N*N!)
			featureConstructors.a = sqrt(2.^(featureConstructors.N+1)./(factorial(featureConstructors.N).*kArgs.^(featureConstructors.N)));
		otherwise,
			fprintf('Unknown Kernel type\n');
			keyboard;
	end
	
	% Find out how many feature constructors do we need
	featureConstructors.numFeatureConstructors = sum(featureConstructors.N);
	featureConstructors.W = 2*randi([0 1],featureConstructors.d,featureConstructors.numFeatureConstructors) - 1;
end