#include "mex.h"

#define IS_INT_SCALAR(P) (mxIsInt32(P) && mxGetNumberOfElements(P) == 1)

void mexFunction(int nlhs, mxArray *plhs[], int rhs, const mxArray *prhs[]){
	int8_t *FC;						/* The feature constructors */
	double *FIn, *FOut;				/* The features - input as well output - are real valued */
	int d, D, numFC, numF;			/* Original and final dimensionality, number of feature constructors and features */
	int *compFC, *temp;				/* Complexity of the feature constructors and a temporary storage */
	int dCounter, DCounter, compFCCounter, fCounter, fcCounter;	/* Counters for various purposes */
	
	#define FC_IN		prhs[0]		/* Feature Constructors */
	#define F_IN		prhs[1]		/* Input Features */
	#define D_IN		prhs[2]		/* How many features do we want */
	#define COMP_FC_IN	prhs[3]		/* What is the complexity of each constructed feature */
	#define F_OUT		plhs[0]		/* Output features */
	
	/* Feature constructors are simply arrays with values in the set {-1,1} - no need to have anything more extravagant to store them */
	if(!mxIsInt8(FC_IN))
		mexErrMsgTxt("Pass in only a signed 8-bit integer arrays as feature constructor\n");
		
	/* The paramter D_IN should be a scalar integer */
	if(!IS_INT_SCALAR(D_IN))
		mexErrMsgTxt("Pass in only a single 32-bit integer as final dimensionality\n");
		
	/* The parameter COMP_FC_IN should be an integer array */
	if(!mxIsInt32(COMP_FC_IN))
		mexErrMsgTxt("Pass in only a 32-bit integer vector as feature constructor complexity\n");
		
	d = mxGetM(FC_IN);
		
	/* The dimensionality of the input features and the feature constructors should match to enable dot products to be taken */
	if(d != mxGetM(F_IN))
		mexErrMsgTxt("The dimensionality of the feature constructors does not match that of the features\n");
	
	numF = mxGetN(F_IN);
	numFC = mxGetN(FC_IN);
	temp = (int*) mxGetPr(D_IN);
	D = temp[0];
	
	/* COMP_FC_IN should have as many elements as the number of dimensions requested */
	if(mxGetM(COMP_FC_IN) != D)
		mexErrMsgTxt("Size of feature complexity array does not match the number of final features\n");
	
	compFC = (int*) mxGetPr(COMP_FC_IN);
	
	int checkSum = 0;
	int checkCounter;
	for(checkCounter = 0; checkCounter < D; checkCounter++){
		if(compFC[checkCounter] < 1)
			mexErrMsgTxt("Feature complexities should be positive non-zero integers\n");
		checkSum += compFC[checkCounter];
	}
		
	if(checkSum != numFC)
		mexErrMsgTxt("Feature complexities dont add up to the number of feature constructors passed\n");
		
	FC = (int8_t*) mxGetPr(FC_IN);
	FIn = mxGetPr(F_IN);
	
	/* Return the output features in liblinear format so that no transposes are necessary later on - moreover, exploiting column order is easier this way */
	F_OUT = mxCreateDoubleMatrix(0, 0, mxREAL);
	mxSetM(F_OUT,numF);
	mxSetN(F_OUT,D);
	mxSetData(F_OUT, mxMalloc(sizeof(double)*numF*D));
	
	FOut = mxGetPr(F_OUT);
	
	compFCCounter = 0;
	DCounter = 0;
	
	double dotProduct;
	
	for(fcCounter = 0; fcCounter < numFC; fcCounter++){				/* Use a particular feature constructor on each input feature */
		if(compFCCounter == compFC[DCounter]){						/* We have completely constructed a feature, move on to the next one */
			compFCCounter = 0;
			DCounter++;
		}
		for(fCounter = 0; fCounter < numF; fCounter++){				/* Visit each input feature in turn */
			if(compFCCounter == 0)									/* If we are just starting to construct this output feature then we should initialize its value */
				FOut[fCounter + DCounter*numF] = 1;
				
			dotProduct = 0;
			for(dCounter = 0; dCounter < d; dCounter++){
				/* Just a clever way of avoiding a floating point multiplication and instead doing a floating point addition */
				/* since we know that if we are using this feature constructor then FC must have values in {-1,1} */
				if(FC[dCounter + fcCounter*d] > 0)
					dotProduct += FIn[dCounter + fCounter*d];
				else
					dotProduct -= FIn[dCounter + fCounter*d];
			}
			/*if(dotProduct == 0)
				mexPrintf("Warning : FC number %d gave zero dot product on vector %d\n",fcCounter,fCounter);*/
			FOut[fCounter + DCounter*numF] *= dotProduct;			/* Update this feature with the dot product value */
		}
		compFCCounter++;
	}
	
	return;
}